@@ -5,11 +5,8 @@
 use crate::err::ErrorDetail;
 use derive_builder::Builder;
 use derive_more::AsRef;
-use directories::ProjectDirs;
 use fs_mistrust::{Mistrust, MistrustBuilder};
-use once_cell::sync::Lazy;
 use serde::{Deserialize, Serialize};
-use std::borrow::Cow;
 use std::collections::HashMap;
 use std::path::Path;
 use std::path::PathBuf;
@@ -70,64 +67,6 @@ pub mod vanguards {
     pub use tor_guardmgr::{VanguardConfig, VanguardConfigBuilder};
 }
 
-/// A [`CfgPathResolver`] with the base variables configured for a [`TorClientConfig`].
-///
-/// A [`TorClientConfig`] may set additional variables on its resolver.
-///
-/// This should only be used by `TorClient` users. Libraries should be written in a
-/// resolver-agnostic way (shouldn't rely on resolving `ARTI_CONFIG` for example).
-///
-/// The supported variables are:
-///   * `ARTI_CACHE`: an arti-specific cache directory.
-///   * `ARTI_CONFIG`: an arti-specific configuration directory.
-///   * `ARTI_SHARED_DATA`: an arti-specific directory in the user's "shared
-///     data" space.
-///   * `ARTI_LOCAL_DATA`: an arti-specific directory in the user's "local
-///     data" space.
-///   * `PROGRAM_DIR`: the directory of the currently executing binary.
-///     See documentation for [`std::env::current_exe`] for security notes.
-///   * `USER_HOME`: the user's home directory.
-///
-/// These variables are implemented using the `directories` crate, and
-/// so should use appropriate system-specific overrides under the
-/// hood. (Some of those overrides are based on environment variables.)
-/// For more information, see that crate's documentation.
-fn base_path_resolver() -> CfgPathResolver {
-    let arti_cache = project_dirs().map(|x| Cow::Owned(x.cache_dir().to_owned()));
-    let arti_config = project_dirs().map(|x| Cow::Owned(x.config_dir().to_owned()));
-    let arti_shared_data = project_dirs().map(|x| Cow::Owned(x.data_dir().to_owned()));
-    let arti_local_data = project_dirs().map(|x| Cow::Owned(x.data_local_dir().to_owned()));
-    let program_dir = get_program_dir().map(Cow::Owned);
-    let user_home = tor_config_path::home().map(Cow::Borrowed);
-
-    let mut resolver = CfgPathResolver::default();
-
-    resolver.set_var("ARTI_CACHE", arti_cache);
-    resolver.set_var("ARTI_CONFIG", arti_config);
-    resolver.set_var("ARTI_SHARED_DATA", arti_shared_data);
-    resolver.set_var("ARTI_LOCAL_DATA", arti_local_data);
-    resolver.set_var("PROGRAM_DIR", program_dir);
-    resolver.set_var("USER_HOME", user_home);
-
-    resolver
-}
-
-/// Return the directory holding the currently executing program.
-fn get_program_dir() -> Result<PathBuf, CfgPathError> {
-    let binary = std::env::current_exe().map_err(|_| CfgPathError::NoProgramPath)?;
-    let directory = binary.parent().ok_or(CfgPathError::NoProgramDir)?;
-    Ok(directory.to_owned())
-}
-
-/// Return a ProjectDirs object for the Arti project.
-fn project_dirs() -> Result<&'static ProjectDirs, CfgPathError> {
-    /// lazy cell holding the ProjectDirs object.
-    static PROJECT_DIRS: Lazy<Option<ProjectDirs>> =
-        Lazy::new(|| ProjectDirs::from("org", "torproject", "Arti"));
-
-    PROJECT_DIRS.as_ref().ok_or(CfgPathError::NoProjectDirs)
-}
-
 /// Configuration for client behavior relating to addresses.
 ///
 /// This type is immutable once constructed. To create an object of this type,
@@ -705,7 +644,7 @@ pub struct TorClientConfig {
     #[builder(setter(skip))]
     #[builder_field_attr(serde(skip))]
     #[educe(PartialEq(ignore), Eq(ignore))]
-    #[builder(default = "base_path_resolver()")]
+    #[builder(default = "tor_config_path::arti_client_base_resolver()")]
     pub(crate) path_resolver: CfgPathResolver,
 }
 impl_standard_builder! { TorClientConfig }
@@ -876,7 +815,7 @@ impl TorClientConfigBuilder {
 /// Return the filenames for the default user configuration files
 pub fn default_config_files() -> Result<Vec<ConfigurationSource>, CfgPathError> {
     // the base path resolver includes the 'ARTI_CONFIG' variable
-    let path_resolver = base_path_resolver();
+    let path_resolver = tor_config_path::arti_client_base_resolver();
 
     ["${ARTI_CONFIG}/arti.toml", "${ARTI_CONFIG}/arti.d/"]
         .into_iter()
@@ -1192,59 +1131,4 @@ mod test {
             chk(&from_toml(test_case), *expected);
         }
     }
-
-    fn cfg_variables() -> impl IntoIterator<Item = (&'static str, PathBuf)> {
-        let list = [
-            ("ARTI_CACHE", project_dirs().unwrap().cache_dir()),
-            ("ARTI_CONFIG", project_dirs().unwrap().config_dir()),
-            ("ARTI_SHARED_DATA", project_dirs().unwrap().data_dir()),
-            ("ARTI_LOCAL_DATA", project_dirs().unwrap().data_local_dir()),
-            ("PROGRAM_DIR", &get_program_dir().unwrap()),
-            ("USER_HOME", tor_config_path::home().unwrap()),
-        ];
-
-        list.into_iter()
-            .map(|(a, b)| (a, b.to_owned()))
-            .collect::<Vec<_>>()
-    }
-
-    #[cfg(not(target_family = "windows"))]
-    #[test]
-    fn expand_variables() {
-        // test using the actual resolver from the `TorClientConfig` after it's done all of its
-        // builder stuff, not just `base_path_resolver()`
-        let path_resolver = TorClientConfig::builder().build().unwrap();
-        let path_resolver = path_resolver.as_ref();
-
-        for (var, val) in cfg_variables() {
-            let p = CfgPath::new(format!("${{{var}}}/example"));
-            assert_eq!(p.to_string(), format!("${{{var}}}/example"));
-
-            let expected = val.join("example");
-            assert_eq!(p.path(path_resolver).unwrap().to_str(), expected.to_str());
-        }
-
-        let p = CfgPath::new("${NOT_A_REAL_VAR}/example".to_string());
-        assert!(p.path(path_resolver).is_err());
-    }
-
-    #[cfg(target_family = "windows")]
-    #[test]
-    fn expand_variables() {
-        // test using the actual resolver from the `TorClientConfig` after it's done all of its
-        // builder stuff, not just `base_path_resolver()`
-        let path_resolver = TorClientConfig::builder().build().unwrap();
-        let path_resolver = path_resolver.as_ref();
-
-        for (var, val) in cfg_variables() {
-            let p = CfgPath::new(format!("${{{var}}}\\example"));
-            assert_eq!(p.to_string(), format!("${{{var}}}\\example"));
-
-            let expected = val.join("example");
-            assert_eq!(p.path(path_resolver).unwrap().to_str(), expected.to_str());
-        }
-
-        let p = CfgPath::new("${NOT_A_REAL_VAR}\\example".to_string());
-        assert!(p.path(path_resolver).is_err());
-    }
 }
