@@ -1,8 +1,27 @@
 /**
  * # Arti RPC core library header.
  *
- * (TODO RPC: This is still a work in progress; please don't rely on it
- * being the final API.)
+ * ## Implementation status
+ *
+ * Note: As of Jan 2025, this library, and the Arti RPC system,
+ * are still under active development.
+ * We believe that they are ready to try out, but it is likely
+ * that they still have bugs and design flaws that we'll need to fix.
+ * Please be ready to report issues at
+ * <https://gitlab.torproject.org/tpo/core/arti>.
+ *
+ * We will make an effort to keep API compatibility over time,
+ * but it's possible that we'll need to break things in small ways.
+ * If we do, we will note them in our changelog and our announcements.
+ *
+ * For now, the Arti RPC interface itself provides only limited
+ * functionality.  We will add support for more features over time.
+ *
+ * ## Who should use this library
+ *
+ * You should use this library if you want to write a program that controls Arti
+ * via its RPC interface, and you don't want to write an implementation of the RPC
+ * protocol from scratch.
  *
  * ## What this library does
  *
@@ -15,24 +34,47 @@
  * This library _does not_ do the work of creating well-formed requests,
  * or interpreting the responses.
  *
- * (Note: Despite this library being exposed via a set of C functions,
+ * Despite this library being exposed via a set of C functions,
  * we don't actually expect you to use it from C.  It's probably a better
- * idea to wrap it in a higher-level language and then use it from there.)
+ * idea to wrap it in a higher-level language and then use it from there.
+ *
+ * The `arti_rpc` python package (available from the Arti git repository)
+ * is one example of such a wrapper.
  *
  * ## Using this library
  *
- * TODO RPC Explain better.
+ * ### Making a connection to Arti
  *
- * Your connection to Arti is represented by an `ArtiRpcConn *`.  Use
- * `arti_rpc_connect()` to create one of these.
+ * First, you will need to have Arti running, and configured to use
+ * rpc.  This will eventually be the default behavior, but for now,
+ * make sure that arti was built using the `rpc` cargo feature, and that
+ * the configuration option `rpc.enable` is set to true.
+ *
+ * (For more detailed instructions on how to do this,
+ * and for examples code,
+ * see the Arti RPC book.)
+ * (TODO Add link once there is one.)
+ *
+ * Your connection to Arti is represented by an `ArtiRpcConn *`.
+ * To get one:
+ * - Call `arti_rpc_conn_builder_new()` to make an `ArtiRpcConnBuilder`.
+ * - Configure the `ArtiRpcConnBuilder` as needed,
+ *   to tell it where to find Arti.
+ *   (If you configured Arti as described above,
+ *   and you're running as the same user,
+ *   then no additional configuration should be necessary.)
+ * - Call `arti_rpc_conn_builder_connect()` to try to connect to Arti.
+ *
+ * <!-- TODO: We'll want to have documentation about connect points,
+ *  but it won't go here. -->
  *
  * Once you have a connection, you can sent Arti various requests in
- * JSON format.  See (TODO RPC: Add a link to a list of comments.)
+ * JSON format.  These requests have documentation of their own;
+ * We'll add a link to it once we've figured out where to host it.
+ *
  * Use `arti_rpc_execute()` to send a simple request; the function will
  * return when the request succeeds, or fails.
  *
- * TODO: Explain handles and other APIs once I add those APIs.
- *
  * Except when noted otherwise, all functions in this library are thread-safe.
  *
  * ## Error handling
@@ -522,6 +564,21 @@ ArtiRpcStatus arti_rpc_conn_execute_with_handle(const ArtiRpcConn *rpc_conn,
                                                 ArtiRpcHandle **handle_out,
                                                 ArtiRpcError **error_out);
 
+/**
+ * Attempt to cancel the request on `rpc_conn` with the provided `handle`.
+ *
+ * Note that cancellation _will_ fail if the handle has already been cancelled,
+ * or has already succeeded or failed.
+ *
+ * On success, return `ARTI_RPC_STATUS_SUCCESS`.
+ *
+ * Otherwise return some other status code,
+ * and set `*error_out` (if provided) to a newly allocated error object.
+ */
+ArtiRpcStatus arti_rpc_conn_cancel_handle(const ArtiRpcConn *rpc_conn,
+                                          const ArtiRpcHandle *handle,
+                                          ArtiRpcError **error_out);
+
 /**
  * Wait until some response arrives on an arti_rpc_handle, or until an error occurs.
  *
@@ -558,8 +615,8 @@ ArtiRpcStatus arti_rpc_handle_wait(const ArtiRpcHandle *handle,
 /**
  * Release storage held by an `ArtiRpcHandle`.
  *
- * NOTE, TODO: This does not cancel the request, but that is not guaranteed.
- * Once we implement cancellation, this may behave differently.
+ * NOTE: This does not cancel the underlying request if it is still running.
+ * To cancel a request, use `arti_rpc_conn_cancel_handle`.
  */
 void arti_rpc_handle_free(ArtiRpcHandle *handle);
 
@@ -628,12 +685,6 @@ void arti_rpc_conn_free(ArtiRpcConn *rpc_conn);
  * Therefore, passing it to functions like `getpeername()`
  * may give unexpected results.
  *
- * If `stream_id_out` is provided
- * (or if Arti is configured to return streams optimistically),
- * the data stream may still be connecting
- * when this request returns.
- * (TODO RPC: Document how to wait for it)
- *
  * If `stream_id_out` is provided,
  * the caller is responsible for releasing the ObjectId;
  * Arti will not deallocate it even when the stream is closed.
