@@ -26,6 +26,7 @@ use crate::usage::{SupportedCircUsage, TargetCircUsage};
 use crate::{timeouts, DirInfo, Error, PathConfig, Result};
 
 use retry_error::RetryError;
+use tor_async_utils::mpsc_channel_no_memquota;
 use tor_basic_utils::retry::RetryDelay;
 use tor_config::MutCfg;
 use tor_error::{debug_report, info_report, internal, warn_report, AbsRetryTime, HasRetryTime};
@@ -145,10 +146,11 @@ pub(crate) trait AbstractCircBuilder<R: Runtime>: Send + Sync {
     type Circ: AbstractCirc + Send + Sync;
     /// An opaque type describing how a given circuit will be built.
     /// It may represent some or all of a path-or it may not.
+    //
     // TODO: It would be nice to have this parameterized on a lifetime,
     // and have that lifetime depend on the lifetime of the directory.
     // But I don't think that rust can do that.
-
+    //
     // HACK(eta): I don't like the fact that `MockablePlan` is necessary here.
     type Plan: Send + Debug + MockablePlan;
 
@@ -248,6 +250,21 @@ pub(crate) trait AbstractCircBuilder<R: Runtime>: Send + Sync {
     /// Return a reference to this builder's `VanguardMgr`.
     #[cfg(feature = "vanguards")]
     fn vanguardmgr(&self) -> &Arc<VanguardMgr<R>>;
+
+    /// Replace our state with a new owning state, assuming we have
+    /// storage permission.
+    fn upgrade_to_owned_state(&self) -> Result<()>;
+
+    /// Reload persistent state from disk, if we don't have storage permission.
+    fn reload_state(&self) -> Result<()>;
+
+    /// Return a reference to this builder's `GuardMgr`.
+    fn guardmgr(&self) -> &tor_guardmgr::GuardMgr<R>;
+
+    /// Reconfigure this builder using the latest set of network parameters.
+    ///
+    /// (NOTE: for now, this only affects circuit timeout estimation.)
+    fn update_network_parameters(&self, p: &tor_netdir::params::NetParameters);
 }
 
 /// Enumeration to track the expiration state of a circuit.
@@ -772,19 +789,6 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> AbstractCircMgr<B, R> {
         usage: &TargetCircUsage,
         dir: DirInfo<'_>,
     ) -> Result<(Arc<B::Circ>, CircProvenance)> {
-        /// Return CEIL(a/b).
-        ///
-        /// Requires that a+b is less than usize::MAX.
-        ///
-        /// This can be removed once the MSRV is >= 1.73.0, which is the version
-        /// that stabilized `std::usize::div_ceil`.
-        ///
-        /// # Panics
-        ///
-        /// Panics if b is 0.
-        fn div_ceil(a: usize, b: usize) -> usize {
-            (a + b - 1) / b
-        }
         /// Largest number of "resets" that we will accept in this attempt.
         ///
         /// A "reset" is an internally generated error that does not represent a
@@ -806,7 +810,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> AbstractCircMgr<B, R> {
         // We compute the maximum number of failures by dividing the maximum
         // number of circuits to attempt by the number that will be launched in
         // parallel for each iteration.
-        let max_failures = div_ceil(
+        let max_failures = usize::div_ceil(
             max_tries as usize,
             std::cmp::max(1, self.builder.launch_parallelism(usage)),
         );
@@ -1063,7 +1067,11 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> AbstractCircMgr<B, R> {
         // stream for us to listen on for notification from pending circuits
         // other than those we are pending on.
         let (pending_request, additional_stream) = {
-            let (send, recv) = mpsc::channel(8);
+            // We don't want this queue to participate in memory quota tracking.
+            // There isn't any circuit yet, so there wouldn't be anything to account it to.
+            // If this queue has the oldest data, probably the whole system is badly broken.
+            // Tearing down the whole circuit manager won't help.
+            let (send, recv) = mpsc_channel_no_memquota(8);
             let pending = Arc::new(PendingRequest {
                 usage: usage.clone(),
                 notify: send,
@@ -1528,8 +1536,10 @@ mod test {
     use crate::{Error, IsolationToken, StreamIsolation, TargetCircUsage, TargetPort, TargetPorts};
     use once_cell::sync::Lazy;
     use tor_guardmgr::fallback::FallbackList;
+    use tor_guardmgr::TestConfig;
     use tor_llcrypto::pk::ed25519::Ed25519Identity;
     use tor_netdir::testnet;
+    use tor_persist::TestingStateMgr;
     use tor_rtcompat::SleepProvider;
     use tor_rtmock::{MockRuntime, MockSleepRuntime};
 
@@ -1572,12 +1582,18 @@ mod test {
         }
     }
 
+    fn make_builder<R: Runtime>(runtime: &R) -> FakeBuilder<R> {
+        let state_mgr = TestingStateMgr::new();
+        let guard_config = TestConfig::default();
+        FakeBuilder::new(runtime, state_mgr, &guard_config)
+    }
+
     #[test]
     fn basic_tests() {
         MockRuntime::test_with_various(|rt| async move {
             let rt = MockSleepRuntime::new(rt);
 
-            let builder = FakeBuilder::new(&rt);
+            let builder = make_builder(&rt);
 
             let mgr = Arc::new(AbstractCircMgr::new(
                 builder,
@@ -1663,7 +1679,7 @@ mod test {
 
             // This will fail once, and then completely time out.  The
             // result will be a failure.
-            let builder = FakeBuilder::new(&rt);
+            let builder = make_builder(&rt);
             builder.set(&ports, vec![FakeOp::Fail, FakeOp::Timeout]);
 
             let mgr = Arc::new(AbstractCircMgr::new(
@@ -1689,7 +1705,7 @@ mod test {
             // that we wait for a little over our predicted time because
             // of our wait-for-next-action logic.
             let ports = TargetCircUsage::new_from_ipv4_ports(&[80, 443]);
-            let builder = FakeBuilder::new(&rt);
+            let builder = make_builder(&rt);
             builder.set(
                 &ports,
                 vec![
@@ -1720,7 +1736,7 @@ mod test {
             let ports = TargetCircUsage::new_from_ipv4_ports(&[80, 443]);
 
             // This will fail a the planning stages, a lot.
-            let builder = FakeBuilder::new(&rt);
+            let builder = make_builder(&rt);
             builder.set(&ports, vec![FakeOp::NoPlan; 2000]);
 
             let mgr = Arc::new(AbstractCircMgr::new(
@@ -1741,7 +1757,7 @@ mod test {
             let ports = TargetCircUsage::new_from_ipv4_ports(&[80, 443]);
 
             // This will fail 1000 times, which is above the retry limit.
-            let builder = FakeBuilder::new(&rt);
+            let builder = make_builder(&rt);
             builder.set(&ports, vec![FakeOp::Fail; 1000]);
 
             let mgr = Arc::new(AbstractCircMgr::new(
@@ -1764,7 +1780,7 @@ mod test {
             // The first time this is called, it will build a circuit
             // with the wrong spec.  (A circuit builder should never
             // actually _do_ that, but it's something we code for.)
-            let builder = FakeBuilder::new(&rt);
+            let builder = make_builder(&rt);
             builder.set(
                 &ports,
                 vec![FakeOp::WrongSpec(target_to_spec(
@@ -1791,7 +1807,7 @@ mod test {
 
             // This will fail twice, and then succeed. The result will be
             // a success.
-            let builder = FakeBuilder::new(&rt);
+            let builder = make_builder(&rt);
             builder.set(&ports, vec![FakeOp::Fail, FakeOp::Fail]);
 
             let mgr = Arc::new(AbstractCircMgr::new(
@@ -1821,7 +1837,7 @@ mod test {
     fn isolated() {
         MockRuntime::test_with_various(|rt| async move {
             let rt = MockSleepRuntime::new(rt);
-            let builder = FakeBuilder::new(&rt);
+            let builder = make_builder(&rt);
             let mgr = Arc::new(AbstractCircMgr::new(
                 builder,
                 rt.clone(),
@@ -1912,7 +1928,7 @@ mod test {
             let ports1 = TargetCircUsage::new_from_ipv4_ports(&[80]);
             let ports2 = TargetCircUsage::new_from_ipv4_ports(&[80, 443]);
 
-            let builder = FakeBuilder::new(&rt);
+            let builder = make_builder(&rt);
             builder.set(&ports1, vec![FakeOp::Timeout]);
 
             let mgr = Arc::new(AbstractCircMgr::new(
@@ -1948,7 +1964,7 @@ mod test {
             // sure that a circuit gets built, and then launch two
             // other circuits that will use it.
             let rt = MockSleepRuntime::new(rt);
-            let builder = FakeBuilder::new(&rt);
+            let builder = make_builder(&rt);
             let mgr = Arc::new(AbstractCircMgr::new(
                 builder,
                 rt.clone(),
@@ -1991,7 +2007,7 @@ mod test {
             // Now let's make some circuits -- one dirty, one clean, and
             // make sure that one expires and one doesn't.
             let rt = MockSleepRuntime::new(rt);
-            let builder = FakeBuilder::new(&rt);
+            let builder = make_builder(&rt);
 
             let circuit_timing = CircuitTimingBuilder::default()
                 .max_dirtiness(Duration::from_secs(15))
@@ -2197,7 +2213,7 @@ mod test {
             let netdir = testnet::construct_netdir().unwrap_if_sufficient().unwrap();
             let dirinfo = DirInfo::Directory(&netdir);
 
-            let builder = FakeBuilder::new(&rt);
+            let builder = make_builder(&rt);
 
             for circs in [2, 8].iter() {
                 let mut circlist = CircList::<FakeBuilder<MockRuntime>, MockRuntime>::new();
