@@ -14,7 +14,7 @@
 //! To launch a channel:
 //!
 //!  * Create a TLS connection as an object that implements AsyncRead +
-//!    AsyncWrite, and pass it to a [ChannelBuilder].  This will
+//!    AsyncWrite + StreamOps, and pass it to a [ChannelBuilder].  This will
 //!    yield an [handshake::OutboundClientHandshake] that represents
 //!    the state of the handshake.
 //!  * Call [handshake::OutboundClientHandshake::connect] on the result
@@ -58,6 +58,7 @@ pub const CHANNEL_BUFFER_SIZE: usize = 128;
 mod circmap;
 mod codec;
 mod handshake;
+pub mod kist;
 pub mod padding;
 pub mod params;
 mod reactor;
@@ -72,8 +73,9 @@ use crate::util::oneshot_broadcast;
 use crate::util::ts::AtomicOptTimestamp;
 use crate::{circuit, ClockSkew};
 use crate::{Error, Result};
+use reactor::BoxedChannelStreamOps;
 use safelog::sensitive as sv;
-use std::future::IntoFuture;
+use std::future::{Future, IntoFuture};
 use std::pin::Pin;
 use std::sync::{Mutex, MutexGuard};
 use std::time::Duration;
@@ -84,7 +86,7 @@ use tor_cell::restricted_msg;
 use tor_error::internal;
 use tor_linkspec::{HasRelayIds, OwnedChanTarget};
 use tor_memquota::mq_queue::{self, ChannelSpec as _, MpscSpec};
-use tor_rtcompat::{CoarseTimeProvider, DynTimeProvider, SleepProvider};
+use tor_rtcompat::{CoarseTimeProvider, DynTimeProvider, SleepProvider, StreamOps};
 
 /// Imports that are re-exported pub if feature `testing` is enabled
 ///
@@ -100,7 +102,7 @@ pub use testing_exports::*;
 #[cfg(not(feature = "testing"))]
 use testing_exports::*;
 
-use asynchronous_codec as futures_codec;
+use asynchronous_codec;
 use futures::channel::mpsc;
 use futures::io::{AsyncRead, AsyncWrite};
 use oneshot_fused_workaround as oneshot;
@@ -119,6 +121,8 @@ use crate::channel::unique_id::CircUniqIdContext;
 pub(crate) use codec::CodecError;
 pub use handshake::{OutboundClientHandshake, UnverifiedChannel, VerifiedChannel};
 
+use kist::KistParams;
+
 restricted_msg! {
     /// A channel message that we allow to be sent from a server to a client on
     /// an open channel.
@@ -151,7 +155,7 @@ pub(crate) type OpenChanCellS2C = ChanCell<OpenChanMsgS2C>;
 /// Type alias: A Sink and Stream that transforms a TLS connection into
 /// a cell-based communication mechanism.
 type CellFrame<T> =
-    futures_codec::Framed<T, crate::channel::codec::ChannelCodec<OpenChanMsgS2C, AnyChanMsg>>;
+    asynchronous_codec::Framed<T, crate::channel::codec::ChannelCodec<OpenChanMsgS2C, AnyChanMsg>>;
 
 /// An open client channel, ready to send and receive Tor cells.
 ///
@@ -193,7 +197,7 @@ pub struct Channel {
     ///
     /// Awaiting will return a `CancelledError` event when the reactor is dropped.
     /// Read to decide if operations may succeed, and is returned by `wait_for_close`.
-    reactor_closed_rx: oneshot_broadcast::Receiver<void::Void>,
+    reactor_closed_rx: oneshot_broadcast::Receiver<Result<CloseInfo>>,
 
     /// A unique identifier for this channel.
     unique_id: UniqId,
@@ -296,7 +300,7 @@ pub(crate) struct ChannelSender {
     /// MPSC sender to send cells.
     cell_tx: mq_queue::Sender<AnyChanCell, mq_queue::MpscSpec>,
     /// A receiver used to check if the channel is closed.
-    reactor_closed_rx: oneshot_broadcast::Receiver<void::Void>,
+    reactor_closed_rx: oneshot_broadcast::Receiver<Result<CloseInfo>>,
     /// Unique ID for this channel. For logging.
     unique_id: UniqId,
 }
@@ -426,7 +430,7 @@ impl ChannelBuilder {
         memquota: ChannelAccount,
     ) -> OutboundClientHandshake<T, S>
     where
-        T: AsyncRead + AsyncWrite + Send + Unpin + 'static,
+        T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
         S: CoarseTimeProvider + SleepProvider,
     {
         handshake::OutboundClientHandshake::new(tls, self.target, sleep_prov, memquota)
@@ -444,6 +448,7 @@ impl Channel {
         link_protocol: u16,
         sink: BoxedChannelSink,
         stream: BoxedChannelStream,
+        streamops: BoxedChannelStreamOps,
         unique_id: UniqId,
         peer_id: OwnedChanTarget,
         clock_skew: ClockSkew,
@@ -493,6 +498,7 @@ impl Channel {
             reactor_closed_tx,
             input: futures::StreamExt::fuse(stream),
             output: sink,
+            streamops,
             circs: circmap,
             circ_unique_id_ctx: CircUniqIdContext::new(),
             link_protocol,
@@ -622,6 +628,13 @@ impl Channel {
         Ok(())
     }
 
+    /// Update the KIST parameters.
+    ///
+    /// Returns `Err` if the channel is closed.
+    pub fn reparameterize_kist(&self, kist_params: KistParams) -> Result<()> {
+        Ok(self.send_control(CtrlMsg::KistConfigUpdate(kist_params))?)
+    }
+
     /// Return an error if this channel is somehow mismatched with the
     /// given target.
     pub fn check_match<T: HasRelayIds + ?Sized>(&self, target: &T) -> Result<()> {
@@ -716,12 +729,18 @@ impl Channel {
     /// Return a future that will resolve once this channel has closed.
     ///
     /// Note that this method does not _cause_ the channel to shut down on its own.
-    ///
-    /// TODO: Perhaps this should return some kind of status indication instead
-    /// of just ().
-    #[cfg(feature = "experimental-api")]
-    pub fn wait_for_close(&self) -> impl futures::Future<Output = ()> + Send + Sync + 'static {
-        self.reactor_closed_rx.clone().into_future().map(|_| ())
+    pub fn wait_for_close(
+        &self,
+    ) -> impl Future<Output = StdResult<CloseInfo, ClosedUnexpectedly>> + Send + Sync + 'static
+    {
+        self.reactor_closed_rx
+            .clone()
+            .into_future()
+            .map(|recv| match recv {
+                Ok(Ok(info)) => Ok(info),
+                Ok(Err(e)) => Err(ClosedUnexpectedly::ReactorError(e)),
+                Err(oneshot_broadcast::SenderDropped) => Err(ClosedUnexpectedly::ReactorDropped),
+            })
     }
 
     /// Make a new fake reactor-less channel.  For testing only, obviously.
@@ -806,6 +825,28 @@ impl HasRelayIds for Channel {
     }
 }
 
+/// The status of a channel which was closed successfully.
+///
+/// **Note:** This doesn't have any associated data,
+/// but may be expanded in the future.
+// I can't think of any info we'd want to return to waiters,
+// but this type leaves the possibility open without requiring any backwards-incompatible changes.
+#[derive(Clone, Debug)]
+#[non_exhaustive]
+pub struct CloseInfo;
+
+/// The status of a channel which closed unexpectedly.
+#[derive(Clone, Debug, thiserror::Error)]
+#[non_exhaustive]
+pub enum ClosedUnexpectedly {
+    /// The channel reactor was dropped or panicked before completing.
+    #[error("channel reactor was dropped or panicked before completing")]
+    ReactorDropped,
+    /// The channel reactor had an internal error.
+    #[error("channel reactor had an internal error")]
+    ReactorError(Error),
+}
+
 /// Make some fake channel details (for testing only!)
 #[cfg(any(test, feature = "testing"))]
 fn fake_channel_details() -> Arc<ChannelDetails> {
