@@ -18,7 +18,7 @@ use tor_cell::chancell::msg::{Destroy, DestroyReason, PaddingNegotiate};
 use tor_cell::chancell::ChanMsg;
 use tor_cell::chancell::{msg::AnyChanMsg, AnyChanCell, CircId};
 use tor_memquota::mq_queue;
-use tor_rtcompat::SleepProvider;
+use tor_rtcompat::{SleepProvider, StreamOps};
 
 use futures::channel::mpsc;
 use oneshot_fused_workaround as oneshot;
@@ -28,13 +28,15 @@ use futures::stream::Stream;
 use futures::Sink;
 use futures::StreamExt as _;
 use futures::{select, select_biased};
-use tor_error::internal;
+use tor_error::{error_report, internal};
 
 use std::fmt;
 use std::pin::Pin;
 use std::sync::Arc;
 
-use crate::channel::{codec::CodecError, padding, params::*, unique_id, ChannelDetails};
+use crate::channel::{
+    codec::CodecError, kist::KistParams, padding, params::*, unique_id, ChannelDetails, CloseInfo,
+};
 use crate::circuit::{celltypes::CreateResponse, CircuitRxSender};
 use tracing::{debug, trace};
 
@@ -45,6 +47,8 @@ pub(super) type BoxedChannelStream = Box<
 /// A boxed trait object that can sink `ChanCell`s.
 pub(super) type BoxedChannelSink =
     Box<dyn Sink<AnyChanCell, Error = CodecError> + Send + Unpin + 'static>;
+/// A boxed trait object that can provide additional `StreamOps` on a `BoxedChannelStream`.
+pub(super) type BoxedChannelStreamOps = Box<dyn StreamOps + Send + Unpin + 'static>;
 /// The type of a oneshot channel used to inform reactor users of the result of an operation.
 pub(super) type ReactorResultChannel<T> = oneshot::Sender<Result<T>>;
 
@@ -87,6 +91,12 @@ pub enum CtrlMsg {
     /// These updates are done via a control message to avoid adding additional branches to the
     /// main reactor `select!`.
     ConfigUpdate(Arc<ChannelPaddingInstructionsUpdates>),
+    /// Enable/disable/reconfigure KIST.
+    ///
+    /// Like in the case of `ConfigUpdate`,
+    /// the sender of these messages is responsible for the optimisation of
+    /// ensuring that "no-change" messages are elided.
+    KistConfigUpdate(KistParams),
 }
 
 /// Object to handle incoming cells and background tasks on a channel.
@@ -99,11 +109,7 @@ pub struct Reactor<S: SleepProvider> {
     pub(super) control: mpsc::UnboundedReceiver<CtrlMsg>,
     /// A oneshot sender that is used to alert other tasks when this reactor is
     /// finally dropped.
-    ///
-    /// It is a sender for Void because we never actually want to send anything here;
-    /// we only want to generate canceled events.
-    #[allow(dead_code)] // the only purpose of this field is to be dropped.
-    pub(super) reactor_closed_tx: oneshot_broadcast::Sender<void::Void>,
+    pub(super) reactor_closed_tx: oneshot_broadcast::Sender<Result<CloseInfo>>,
     /// A receiver for cells to be sent on this reactor's sink.
     ///
     /// `Channel` objects have a sender that can send cells here.
@@ -116,6 +122,8 @@ pub struct Reactor<S: SleepProvider> {
     ///
     /// This should also be backed by a TLS connection if you want it to be secure.
     pub(super) output: BoxedChannelSink,
+    /// A handler for setting stream options on the underlying stream.
+    pub(super) streamops: BoxedChannelStreamOps,
     /// Timer tracking when to generate channel padding
     pub(super) padding_timer: Pin<Box<padding::Timer<S>>>,
     /// Outgoing cells introduced at the channel reactor
@@ -182,6 +190,9 @@ impl<S: SleepProvider> Reactor<S> {
             }
         };
         debug!("{}: Reactor stopped: {:?}", &self, result);
+        // Inform any waiters that the channel has closed.
+        let close_msg = result.as_ref().map_err(Clone::clone).map(|()| CloseInfo);
+        self.reactor_closed_tx.send(close_msg);
         result
     }
 
@@ -306,6 +317,7 @@ impl<S: SleepProvider> Reactor<S> {
                     self.special_outgoing.padding_negotiate = Some(padding_negotiate.clone());
                 }
             }
+            CtrlMsg::KistConfigUpdate(kist) => self.apply_kist_params(&kist),
         }
         Ok(())
     }
@@ -462,21 +474,50 @@ impl<S: SleepProvider> Reactor<S> {
             self.details.unused_since.clear();
         }
     }
+
+    /// Use the new KIST parameters.
+    #[cfg(target_os = "linux")]
+    fn apply_kist_params(&self, params: &KistParams) {
+        use super::kist::KistMode;
+
+        let set_tcp_notsent_lowat = |v: u32| {
+            if let Err(e) = self.streamops.set_tcp_notsent_lowat(v) {
+                // This is bad, but not fatal: not setting the KIST options
+                // comes with a performance penalty, but we don't have to crash.
+                error_report!(e, "Failed to set KIST socket options");
+            }
+        };
+
+        match params.kist_enabled() {
+            KistMode::TcpNotSentLowat => set_tcp_notsent_lowat(params.tcp_notsent_lowat()),
+            KistMode::Disabled => set_tcp_notsent_lowat(u32::MAX),
+        }
+    }
+
+    #[cfg(not(target_os = "linux"))]
+    fn apply_kist_params(&self, params: &KistParams) {
+        use super::kist::KistMode;
+
+        if params.kist_enabled() != KistMode::Disabled {
+            tracing::warn!("KIST not currently supported on non-linux platforms");
+        }
+    }
 }
 
 #[cfg(test)]
 pub(crate) mod test {
     #![allow(clippy::unwrap_used)]
     use super::*;
-    use crate::channel::UniqId;
+    use crate::channel::{ClosedUnexpectedly, UniqId};
     use crate::circuit::CircParameters;
     use crate::fake_mpsc;
     use crate::util::fake_mq;
     use futures::sink::SinkExt;
     use futures::stream::StreamExt;
     use futures::task::SpawnExt;
+    use tor_cell::chancell::msg;
     use tor_linkspec::OwnedChanTarget;
-    use tor_rtcompat::Runtime;
+    use tor_rtcompat::{NoOpStreamOpsHandle, Runtime};
 
     type CodecResult = std::result::Result<OpenChanCellS2C, CodecError>;
 
@@ -501,10 +542,12 @@ pub(crate) mod test {
             trace!("got sink error: {:?}", e);
             CodecError::DecCell(tor_cell::Error::ChanProto("dummy message".into()))
         });
+        let stream_ops = NoOpStreamOpsHandle::default();
         let (chan, reactor) = crate::channel::Channel::new(
             link_protocol,
             Box::new(send1),
             Box::new(recv2),
+            Box::new(stream_ops),
             unique_id,
             dummy_target,
             crate::ClockSkew::None,
@@ -597,7 +640,6 @@ pub(crate) mod test {
         use tor_rtcompat::SleepProvider;
 
         tor_rtcompat::test_with_all_runtimes!(|rt| async move {
-            use tor_cell::chancell::msg;
             let (chan, mut reactor, mut output, mut input) = new_reactor(rt.clone());
 
             let (ret, reac) = futures::join!(chan.new_circ(), reactor.run_once());
@@ -648,7 +690,6 @@ pub(crate) mod test {
     #[test]
     fn bad_cells() {
         tor_rtcompat::test_with_all_runtimes!(|rt| async move {
-            use tor_cell::chancell::msg;
             let (_chan, mut reactor, _output, mut input) = new_reactor(rt);
 
             // shouldn't get created2 cells for nonexistent circuits
@@ -687,7 +728,6 @@ pub(crate) mod test {
         tor_rtcompat::test_with_all_runtimes!(|rt| async move {
             use crate::circuit::celltypes::ClientCircChanMsg;
             use oneshot_fused_workaround as oneshot;
-            use tor_cell::chancell::msg;
 
             let (_chan, mut reactor, _output, mut input) = new_reactor(rt);
 
@@ -728,9 +768,9 @@ pub(crate) mod test {
                 .unwrap();
             let e = reactor.run_once().await.unwrap_err().unwrap_err();
             assert_eq!(
-            format!("{}", e),
-            "Channel protocol violation: Relay cell on pending circuit before CREATED* received"
-        );
+                format!("{}", e),
+                "Channel protocol violation: Relay cell on pending circuit before CREATED* received"
+            );
 
             // If a relay cell is sent on a non-existent channel, that's an error.
             input
@@ -776,7 +816,6 @@ pub(crate) mod test {
         tor_rtcompat::test_with_all_runtimes!(|rt| async move {
             use crate::circuit::celltypes::*;
             use oneshot_fused_workaround as oneshot;
-            use tor_cell::chancell::msg;
 
             let (_chan, mut reactor, _output, mut input) = new_reactor(rt);
 
@@ -858,6 +897,11 @@ pub(crate) mod test {
             assert!(!chan.is_closing());
             drop(reactor);
             assert!(chan.is_closing());
+
+            assert!(matches!(
+                chan.wait_for_close().await,
+                Err(ClosedUnexpectedly::ReactorDropped),
+            ));
         });
     }
 
@@ -873,6 +917,34 @@ pub(crate) mod test {
             let r = reactor.run().await;
             assert!(r.is_ok());
             assert!(chan.is_closing());
+
+            assert!(chan.wait_for_close().await.is_ok());
+        });
+    }
+
+    #[test]
+    fn reactor_error_wait_for_close() {
+        tor_rtcompat::test_with_all_runtimes!(|rt| async move {
+            let (chan, reactor, _output, mut input) = new_reactor(rt);
+
+            // force an error by sending created2 cell for nonexistent circuit
+            let created2_cell = msg::Created2::new(*b"hihi").into();
+            input
+                .send(Ok(OpenChanCellS2C::new(CircId::new(7), created2_cell)))
+                .await
+                .unwrap();
+
+            // `reactor.run()` should return an error
+            let run_error = reactor.run().await.unwrap_err();
+
+            // `chan.wait_for_close()` should return the same error
+            let Err(ClosedUnexpectedly::ReactorError(wait_error)) = chan.wait_for_close().await
+            else {
+                panic!("Expected a 'ReactorError'");
+            };
+
+            // `Error` doesn't implement `PartialEq`, so best we can do is to compare the strings
+            assert_eq!(run_error.to_string(), wait_error.to_string());
         });
     }
 }
