@@ -10,9 +10,7 @@ use tor_cell::relaycell::{AnyRelayMsgOuter, RelayMsg, UnparsedRelayMsg};
 use crate::crypto::cell::HopNum;
 use crate::{Error, Result};
 
-use std::task::Context;
-
-use super::{ConversationInHandler, MetaCellDisposition};
+use super::MetaCellDisposition;
 
 /// An object that checks whether incoming control messages are acceptable on a
 /// circuit, and delivers them to a client if so.
@@ -40,11 +38,7 @@ pub trait MsgHandler {
     /// or highly contended locks, to avoid blocking the circuit reactor.
     ///
     /// If this function returns an error, the circuit will be closed.
-    fn handle_msg(
-        &mut self,
-        conversation: ConversationInHandler<'_, '_, '_>,
-        msg: AnyRelayMsg,
-    ) -> Result<MetaCellDisposition>;
+    fn handle_msg(&mut self, msg: AnyRelayMsg) -> Result<MetaCellDisposition>;
 }
 
 /// Wrapper for `MsgHandler` to implement `MetaCellHandler`
@@ -70,9 +64,8 @@ impl<T: MsgHandler + Send> super::reactor::MetaCellHandler for UserMsgHandler<T>
 
     fn handle_msg(
         &mut self,
-        cx: &mut Context<'_>,
         msg: UnparsedRelayMsg,
-        reactor: &mut super::reactor::Reactor,
+        _reactor: &mut super::reactor::Reactor,
     ) -> Result<MetaCellDisposition> {
         let cell: AnyRelayMsgOuter = msg.decode().map_err(|err| Error::BytesErr {
             object: "cell for message handler",
@@ -85,11 +78,6 @@ impl<T: MsgHandler + Send> super::reactor::MetaCellHandler for UserMsgHandler<T>
                 msg.cmd()
             )));
         }
-        let conversation = ConversationInHandler {
-            reactor,
-            cx,
-            hop_num: self.hop,
-        };
-        self.handler.handle_msg(conversation, msg)
+        self.handler.handle_msg(msg)
     }
 }
